#!/usr/bin/python

import glob
import os
import sys
import argparse
import fileinput
import subprocess

parser = argparse.ArgumentParser(description="Prepare the CRP potential energy surface.")
subparsers = parser.add_subparsers(help="commands", dest="command")

patchparser = subparsers.add_parser("patch", help="apply a patch file to a dataset")

diffparser = subparsers.add_parser("diff", help="create a patch file from a dataset")

interpolatesplineparser = subparsers.add_parser("interpolatespline", help="interpolate a point with spline interpolation")
interpolatesplineparser.add_argument("--r", action="store", help="perform interpolation for r=...")
interpolatesplineparser.add_argument("--z", action="store", help="perform interpolation for z=...")
interpolatesplineparser.add_argument("--omit", action="store", help="omit these points")
interpolatesplineparser.add_argument("--interpolationfunction", action="store_true", help="use interpolation function")
interpolatesplineparser.add_argument("crp_directory", help="where the .2d files are stored")
interpolatesplineparser.add_argument("file", help="the .2d file to process")
interpolatesplineparser.add_argument("value", default="", help="at which point to evaluate the spline")
#interpolatesplineparser.
#interpolatesplineparser.

#parser.add_argument("--interpolatespline", action="store_true", help="interpolate a point with spline interpolation")
#parser.add_argument("--r", action="store_true", help="perform interpolation over r")
#parser.add_argument("--z", action="store_true", help="perform interpolation over z")
#parser.add_argument("--strip", action="store_true", help="remove a patch from a dataset")
#parser.add_argument("--shift", action="store", help="shift energies in patch file by this amount")
parser.add_argument("directory", nargs="?", default=".", help="the directory to work in")

args = parser.parse_args()

#print args

files = glob.glob( args.directory + "/Molecule/*.2d" )

#sys.exit()

#if not args.diff and not args.patch:
#	print "Please specify either --diff or --patch"
#	sys.exit()

if args.command == "diff":
	for i in files:
		f = open( i, "r" )
		lines = f.readlines()
		for j in lines:
			if "#" in j: # These are the modified lines
				stripfile = i.split("_")
				z = j.strip().split()[0]
				r = j.strip().split()[1]
				v = j.strip().split()[2]
				print stripfile[len(stripfile)-1], z, r, v
		f.close()

if args.command == "patch":
	for line in sys.stdin:
		patchparts = line.strip().split()
		file = patchparts[0]
		patchz = patchparts[1]
		patchr = patchparts[2]
		patchv = patchparts[3]
		for i in files:
			if file in i:
				f = open( i, "r" )
				lines = f.readlines()
				f.close()
				f = open( i, "w" )
				for j in lines:
					parts = j.strip().split()
					z = parts[0]
					r = parts[1]
					v = parts[2]
					origv = parts[len(parts)-1]
					if patchz == z and patchr == r:
						f.write( " ".join([z, r, patchv, "#", origv, "\n"]) )
					else:
						f.write( j )
				f.close()

if args.command == "interpolatespline":
	# Do some checks
	if args.omit:
		omit = [ float(x) for x in args.omit.split(";") ]
	else:
		omit = []
	value = float( args.value )
	if ( args.z and args.r ) or ( (not args.z) and (not args.r) ):
		print "Not specified direction, stopping..."
		sys.exit(1)
	if args.r:
		args.r = float( args.r )
	if args.z:
		args.z = float( args.z )
	# Gather points
	f = open( args.crp_directory + "/Molecule/" + args.file )
	lines = [ x.strip().split() for x in f.readlines()[2:] ]
	f.close()
	f = open( args.file + ".int" )
	intlines = [ x.strip().split() for x in f.readlines() if x.strip() != "" ]
	f.close()
#	print lines
#	print
#	print
#	print intlines
	# Now check them
	newlines = []
	newintlines = []
	# keep r fixed
	if args.r:
		for x in lines:
			if abs( float( x[1] ) - args.r ) < 1e-6:
				skip = False
				for y in omit:
					if abs( float( x[0] ) - y ) < 1e-6:
						skip = True
				if not skip:
					newlines.append( x )
		for x in intlines:
			if abs( float( x[1] ) - args.r ) < 1e-6:
				skip = False
				for y in omit:
					if abs( float( x[0] ) - y ) < 1e-6:
						skip = True
				if not skip:
					newintlines.append( x )
	# keep z fixed
	if args.z:
		for x in lines:
			if abs( float( x[0] ) - args.z ) < 1e-6:
				skip = False
				for y in omit:
					if abs( float( x[1] ) - y ) < 1e-6:
						skip = True
				if not skip:
					newlines.append( x )
		for x in intlines:
			if abs( float( x[0] ) - args.z ) < 1e-6:
				skip = False
				for y in omit:
					if abs( float( x[1] ) - y ) < 1e-6:
						skip = True
				if not skip:
					newintlines.append( x )
	print newlines
	print newintlines
	# Do we interpolate the interpolation function or the full potential?
	temp = []
	if args.interpolationfunction:
		if args.r:
			for x in newintlines:
				temp.append( str(x[0]) + " " + str(x[2]) + "\n" )
		if args.z:
			for x in newintlines:
				temp.append( str(x[1]) + " " + str(x[2]) + "\n" )
	else:
		if args.r:
			for x in newlines:
				temp.append( str(x[0]) + " " + str(x[2]) + "\n" )
		if args.z:
			for x in newlines:
				temp.append( str(x[1]) + " " + str(x[2]) + "\n" )
	f = open( "FILE.1d", "w" )
	f.writelines( temp )
	f.close()
#	print newintlines
	# Start our "external" spline code
	proc = subprocess.Popen( "/home/guido/work-PhD/CRP-tools/Interpolation_Tools/Int_Spline.x " + args.value, shell=True, stdout=subprocess.PIPE )
	output, error = proc.communicate()
	next = False
	number = ""
	for line in output.split("\n"):
		if next:
			next = False
			number = float( line.strip() )
		if "The spline evaluated at" in line:
			next = True
	if args.z:
		print "The interpolation has been done for r = " + str( args.value )
	if args.r:
		print "The interpolation has been done for Z = " + str( args.value )
	print
	if args.interpolationfunction:
		print "At this point, the interpolation function by spline interpolation is " + str( number )
		if args.z:
			origval = 0.0
			for x in intlines:
				if abs( float( x[0] ) - float( args.z ) ) < 1e-6 and abs( float( x[1] ) - float( args.value ) ) < 1e-6:
					print "Originally this was " + str( x[2] )
					origval = float( x[2] )
			origvalfull = 0.0
			for x in lines:
				if abs( float( x[0] ) - float( args.z ) ) < 1e-6 and abs( float( x[1] ) - float( args.value ) ) < 1e-6:
					print "And the full potential was " + str( x[2] )
					origvalfull = float( x[2] )
		if args.r:
			origval = 0.0
			for x in intlines:
				if abs( float( x[1] ) - float( args.r ) ) < 1e-6 and abs( float( x[0] ) - float( args.value ) ) < 1e-6:
					print "Originally this was " + str( x[2] )
					origval = float( x[2] )
			origvalfull = 0.0
			for x in lines:
				if abs( float( x[1] ) - float( args.r ) ) < 1e-6 and abs( float( x[0] ) - float( args.value ) ) < 1e-6:
					print "And the full potential was " + str( x[2] )
					origvalfull = float( x[2] )
		if origval != 0.0 and origvalfull != 0.0:
			print "Which means the new point is going to be " + str( origvalfull - origval + number )
	else:
		print "At this point, the full potential by spline interpolation is " + str( number )

#	print newlines
#	print newintlines
#	print len( lines )
#	print len( intlines )
